//
//  TKVariation.m
//  ThemeKit
//
//  Created by Colin Cornaby on Sun Dec 21 2003.
//  Copyright (c) 2003 Carpe Stellarem. All rights reserved.
//

#import "TKVariation.h"
#import "TKTheme.h"
#import "TKAppSkin.h"

@implementation TKVariation

-(id)appSkinForIdentifier:(NSString *)identifier
{
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(appIdentifier == '%@')", identifier];
	NSArray *variationsFound = [[[self primitiveValueForKey:@"applicationSkins"] allObjects] filteredArrayUsingPredicate:predicate];
	if([variationsFound count]==0)
		return nil;
	return [variationsFound objectAtIndex:0];
}

-(void)removeAppSkin:(id)appSkin
{
	[[self mutableSetValueForKey:@"applicationSkins"] removeObject:appSkin];
}

-(NSArray *)appSkins{
	return [[self valueForKey:@"applicationSkins"] allObjects];
}

-(id)addAppSkin:(id)appSkin
{
	NSMutableSet *appSkinSet = [self mutableSetValueForKey:@"applicationSkins"];
	NSEnumerator *enumerator = [appSkinSet objectEnumerator];
	id value;
	while ((value = [enumerator nextObject])) {
		if([[value appIdentifier] isEqualToString:[appSkin appIdentifier]])
			[appSkinSet removeObject:value];
	}
	[appSkinSet addObject:appSkin];
	return appSkin;
}

-(void)setDescriptorHTML:(NSString *)htmlCode
{
	[self setMetadataForKey:@"TKVariationHTMLCode" forValue:htmlCode];
}

-(NSString *)descriptorHTML
{
	NSString *htmlCode = [self metadataForKey:@"TKVariationHTMLCode"];
	if(htmlCode == nil)
		return @"";
	return htmlCode;
}

-(NSDictionary *)descriptorFileDictionary
{
	return [[self metadataForKey:@"TKDescriptorFileDictionary"] propertyList];
}

-(void)setDescriptorFileDictionary:(NSDictionary *)descriptorFileDictionary
{
	[self setMetadataForKey:@"TKDescriptorFileDictionary" forValue:[descriptorFileDictionary description]];
}

- (id)initWithEntity:(NSEntityDescription*)entity insertIntoManagedObjectContext:(NSManagedObjectContext*)context
{
	self = [super initWithEntity:(NSEntityDescription*)entity insertIntoManagedObjectContext:(NSManagedObjectContext*)context];
	if([self metadataForKey:@"TKDRMKey"])
		return NULL;
	idCache = [[NSMutableDictionary alloc] init];
	return self;
}

-(void)dealloc
{
	[idCache release];
	[super dealloc];
}

-(void)setDRMPublisher:(NSString *)publisher
{
	[self setMetadataForKey:@"TKDRMPublisher" forValue:publisher];
}

-(void)setName:(NSString *)theName
{
	[self setMetadataForKey:@"TKFlavorName" forValue:theName];
}

-(NSString *)name{
	NSString *flavorName = [self metadataForKey:@"TKFlavorName"];
	if(flavorName==nil)
		return @"Untitled Theme";
	return flavorName;
}
@end
